@@ -2,7 +2,7 @@
 
 from textwrap import indent
 
-import borg.archiver
+import borg.archiver  # noqa: F401 - need import to get Error and ErrorWithTraceback subclasses.
 from borg.helpers import Error, ErrorWithTraceback
 
 classes = Error.__subclasses__() + ErrorWithTraceback.__subclasses__()
