@@ -147,7 +147,7 @@ def test_obfuscate():
     # 2 id bytes compression, 2 id bytes obfuscator. 4 length bytes
     assert len(data) + 8 <= len(compressed) <= len(data) * 101 + 8
     # compressing 100 times the same data should give at least 50 different result sizes
-    assert len(set(len(compressor.compress(data)) for i in range(100))) > 50
+    assert len({len(compressor.compress(data)) for i in range(100)}) > 50
 
     cs = CompressionSpec('obfuscate,2,lz4')
     assert isinstance(cs.inner.compressor, LZ4)
@@ -158,7 +158,7 @@ def test_obfuscate():
     min_compress, max_compress = 0.2, 0.001  # estimate compression factor outer boundaries
     assert max_compress * len(data) + 8 <= len(compressed) <= min_compress * len(data) * 1001 + 8
     # compressing 100 times the same data should give multiple different result sizes
-    assert len(set(len(compressor.compress(data)) for i in range(100))) > 10
+    assert len({len(compressor.compress(data)) for i in range(100)}) > 10
 
     cs = CompressionSpec('obfuscate,6,zstd,3')
     assert isinstance(cs.inner.compressor, ZSTD)
@@ -169,7 +169,7 @@ def test_obfuscate():
     min_compress, max_compress = 0.2, 0.001  # estimate compression factor outer boundaries
     assert max_compress * len(data) + 8 <= len(compressed) <= min_compress * len(data) * 10000001 + 8
     # compressing 100 times the same data should give multiple different result sizes
-    assert len(set(len(compressor.compress(data)) for i in range(100))) > 90
+    assert len({len(compressor.compress(data)) for i in range(100)}) > 90
 
     cs = CompressionSpec('obfuscate,2,auto,zstd,10')
     assert isinstance(cs.inner.compressor, Auto)
@@ -180,7 +180,7 @@ def test_obfuscate():
     min_compress, max_compress = 0.2, 0.001  # estimate compression factor outer boundaries
     assert max_compress * len(data) + 8 <= len(compressed) <= min_compress * len(data) * 1001 + 8
     # compressing 100 times the same data should give multiple different result sizes
-    assert len(set(len(compressor.compress(data)) for i in range(100))) > 10
+    assert len({len(compressor.compress(data)) for i in range(100)}) > 10
 
     cs = CompressionSpec('obfuscate,110,none')
     assert isinstance(cs.inner.compressor, CNONE)
@@ -188,11 +188,13 @@ def test_obfuscate():
     data = bytes(1000)
     compressed = compressor.compress(data)
     # N blocks + 2 id bytes obfuscator. 4 length bytes
-    assert 1000 + 6 <= len(compressed) <= 1000 + 6 + 1024
+    # The 'none' compressor also adds 2 id bytes
+    assert 6 + 2 + 1000 <= len(compressed) <= 6 + 2 + 1000 + 1024
     data = bytes(1100)
     compressed = compressor.compress(data)
     # N blocks + 2 id bytes obfuscator. 4 length bytes
-    assert 1100 + 6 <= len(compressed) <= 1100 + 6 + 1024
+    # The 'none' compressor also adds 2 id bytes
+    assert 6 + 2 + 1100 <= len(compressed) <= 6 + 2 + 1100 + 1024
 
 
 def test_compression_specs():
